﻿using Steema.TeeChart;
using Steema.TeeChart.Styles;
using System;
using System.IO;
using Xamarin.Forms;

namespace GaugeTestApp
{
    public class SemiCircleGauge : ChartView
    {
        #region Variables

        private int _count = 0;

        #endregion

        #region Properties

        public CircularGauge CircularGauge { get; private set; }
        public string ImageFileName { get; set; }

        #endregion

        #region ctor

        public SemiCircleGauge()
        {
            CreateGauge();
        }

        private void CreateGauge()
        {
            ChartView cv = this;

            cv.VerticalOptions = LayoutOptions.FillAndExpand;
            cv.HorizontalOptions = LayoutOptions.FillAndExpand;

            cv.Chart.Aspect.View3D = false;

            // ### CHART LEGEND ###
            cv.Chart.Legend.Visible = false;

            // ### CHART HEADER ###
            cv.Chart.Header.Text = "";
            cv.Chart.Header.Visible = true;
            cv.Chart.Header.Height = 50;
            cv.Chart.Header.Alignment = TextAlignment.Center;

            // ### CHART FOOTER ###
            cv.Chart.Footer.Text = "";
            cv.Chart.Footer.Visible = false;
            cv.Chart.Footer.Height = 0;
            cv.Chart.Footer.Alignment = TextAlignment.Center;

            // ### PANEL ###
            cv.Chart.Panel.Gradient.Visible = false;
            cv.Chart.Panel.Color = Color.Transparent;
            cv.Chart.Panel.MarginUnits = PanelMarginUnits.Percent;
            cv.Chart.Panel.MarginTop = 2;
            cv.Chart.Panel.MarginBottom = 2;
            cv.Chart.Panel.MarginRight = 2;
            cv.Chart.Panel.MarginLeft = 2;

            // ### CIRCULAR GAUGE ###
            CircularGauge = new CircularGauge();
            cv.Chart.Series.Add(CircularGauge);
            CircularGauge.Title = "";

            // ### TICKS ###
            CircularGauge.MinorTicks.Visible = false;
            CircularGauge.Ticks.Visible = true;
            CircularGauge.Ticks.Color = Color.Black;
            CircularGauge.Ticks.HorizSize = 2;
            CircularGauge.Ticks.VertSize = 65;
            CircularGauge.Ticks.Position = 85;

            // ### HANDS ###
            CircularGauge.HandDistance = 95;
            CircularGauge.HandOffset = 20;
            CircularGauge.Hand.HorizSize = 10;
            CircularGauge.Hand.Color = Color.FromRgb(150, 50, 10);
            CircularGauge.Hand.Draw3D = false;
            CircularGauge.Hand.Pen.Visible = false;
            CircularGauge.Hand.Gradient.Visible = false;
            CircularGauge.Hand.Shadow.Visible = true;
            CircularGauge.Hand.Style = PointerStyles.DownTriangle;

            // ### CENTER ###
            CircularGauge.Center.Color = Color.FromRgb(70, 130, 180); // SteelBlue #4682B4
            CircularGauge.Center.Gradient.Visible = false;
            CircularGauge.Center.HorizSize = 20;
            CircularGauge.Center.VertSize = 20;
            CircularGauge.Center.Draw3D = false;
            CircularGauge.Center.Pen.Visible = false;
            CircularGauge.Center.Shadow.Visible = false;

            // ### FACE ###
            CircularGauge.FaceBrush.Visible = true;
            CircularGauge.FaceBrush.Solid = true;
            CircularGauge.FaceBrush.Color = Color.Transparent;
            CircularGauge.FaceBrush.Gradient.Visible = false;

            // ### LINES ###
            int linePosition = 70;
            int lineHorizSize = 40;

            // ### RED LINE ###
            CircularGauge.RedLine.Visible = false;
            CircularGauge.RedLine.Gradient.Visible = true;
            CircularGauge.RedLine.Gradient.UseMiddle = true;
            CircularGauge.RedLine.Gradient.StartColor = Color.Red;
            CircularGauge.RedLine.Gradient.EndColor = Color.LightGreen;
            CircularGauge.RedLine.Gradient.Direction = Steema.TeeChart.Drawing.GradientDirection.FromBottomLeft;
            CircularGauge.RedLine.Position = linePosition;
            CircularGauge.RedLine.Spiralled = false;
            CircularGauge.RedLine.InflateMargins = true;
            CircularGauge.RedLine.HorizSize = lineHorizSize;

            // ### GREEN LINE ###
            CircularGauge.GreenLine.Visible = false;

            // ### FRAME ###
            CircularGauge.Frame.Visible = true;
            CircularGauge.Frame.FrameElementPercents = new double[3] { 30, 40, 30 }; // Outer, Inner, Middle
            CircularGauge.Frame.Circled = true;
            CircularGauge.Frame.Width = 5;
            CircularGauge.Frame.RotationAngle = 0;
            CircularGauge.Frame.TotalAngle = 360;
            // Inner Band
            CircularGauge.Frame.InnerBand.Visible = true;
            CircularGauge.Frame.InnerBand.Solid = true;
            CircularGauge.Frame.InnerBand.Gradient.Visible = false;
            CircularGauge.Frame.InnerBand.Color = Color.White;
            // Middle Band
            CircularGauge.Frame.MiddleBand.Visible = true;
            CircularGauge.Frame.MiddleBand.Solid = true;
            CircularGauge.Frame.MiddleBand.Gradient.Visible = false;
            CircularGauge.Frame.MiddleBand.Color = Color.Transparent;
            // Outer Band
            CircularGauge.Frame.OuterBand.Visible = true;
            CircularGauge.Frame.OuterBand.Solid = true;
            CircularGauge.Frame.OuterBand.Gradient.Visible = false;
            CircularGauge.Frame.OuterBand.Color = Color.White;

            // ### AXIS ###
            CircularGauge.Axis.Labels.Visible = false;
            SetIncrement(0.5);

            // ### VALUE INFO ###
            SetMinMax(0.0D, 1.0D);
            CircularGauge.Value = 0.0;

            // ### ANGLES ###
            CircularGauge.DisplayTotalAngle = 360;
            CircularGauge.DisplayRotationAngle = 0;
            CircularGauge.TotalAngle = 180;

            cv.Chart.AfterDraw += Chart_AfterDraw;
        }

        #endregion

        #region Draw

        private void Chart_AfterDraw(object sender, Steema.TeeChart.Drawing.Graphics3D g)
        {
            try
            {
                _count++;
                System.Diagnostics.Debug.WriteLine($"Chart_AfterDraw > _count = {_count}");

                System.Diagnostics.Debug.WriteLine($"Chart_AfterDraw > Start");
                DoDrawImage(g);
                System.Diagnostics.Debug.WriteLine($"Chart_AfterDraw > End");
            }
            catch (Exception ex)
            {
                System.Diagnostics.Debug.WriteLine(ex.ToString());
            }
        }

        private void DoDrawImage(Steema.TeeChart.Drawing.Graphics3D g)
        {
            System.Diagnostics.Debug.WriteLine("DoDrawImage");

            double imgX = CircularGauge.YRadius / 2;
            double imgY = imgX;

            var img = new Image();
            byte[] resizedImage = DependencyService.Get<IImageResizer>().ResizeImage(ImageFileName, imgX, imgY);
            img.Source = ImageSource.FromStream(() => new MemoryStream(resizedImage));

            var xpos = g.ChartXCenter - (imgX / 2);
            var ypos = g.ChartYCenter + ((CircularGauge.YRadius / 2) - (imgY / 2));
            g.Draw(new Rectangle(xpos, ypos, imgX, imgY), img, Steema.TeeChart.Drawing.ImageMode.Normal, false);
        }

        #endregion

        #region Public Methods

        public void DrawImage()
        {
            DoDrawImage(Chart.Chart.Graphics3D);
        }

        public void SetTitle(string newTitle)
        {
            Chart.Header.Text = newTitle;
        }

        public void SetMinMax(double min, double max)
        {
            CircularGauge.Minimum = min;
            CircularGauge.Maximum = max;
            if (CircularGauge.Value < min)
                CircularGauge.Value = min;
            else if (CircularGauge.Value > max)
                CircularGauge.Value = max;

            CircularGauge.RedLineStartValue = min;
            CircularGauge.RedLineEndValue = max;

        }

        public void SetIncrement(double inc)
        {
            CircularGauge.Axis.Increment = inc;
        }

        #endregion
    }
}